/**
 * creates a queue with a consumer that consumes batches of items
 *
 * note
 * - batches are created when either batchSize is met or delayThreshold is exceeded
 *
 * usecases
 * - sending batches of events to another source, without waiting too long between event creation and event submission
 */
export declare const createQueueWithBatchConsumer: <T>({ consumer, threshold, }: {
    /**
     * the consumer to invoke with batches of items
     */
    consumer: ({ items }: {
        items: T[];
    }) => void | Promise<void>;
    /**
     * the thresholds which will trigger a batch to be consumed
     */
    threshold: {
        /**
         * the milliseconds delay between when an event was added and it is consumed
         *
         * note
         * - this defines the max delay for a batch
         */
        milliseconds: number;
        /**
         * the number of items that are available for consumption
         *
         * note
         * - this will also define the max size for a batch
         */
        size: number;
    };
}) => import("../queue/createQueue").Queue<T>;
